package com.ruoyi.common.utils;

import java.lang.management.ManagementFactory;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.*;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.concurrent.ThreadLocalRandom;

import org.apache.commons.lang3.time.DateFormatUtils;

/**
 * 时间工具类
 * 
 * @author ruoyi
 */
public class DateUtils extends org.apache.commons.lang3.time.DateUtils
{
    public static String YYYY = "yyyy";

    public static String YYYY_MM = "yyyy-MM";

    public static String YYYY_MM_DD = "yyyy-MM-dd";

    public static String YYYYMMDDHHMMSS = "yyyyMMddHHmmss";

    public static String YYYY_MM_DD_HH_MM_SS = "yyyy-MM-dd HH:mm:ss";

    private static String[] parsePatterns = {
            "yyyy-MM-dd", "yyyy-MM-dd HH:mm:ss", "yyyy-MM-dd HH:mm", "yyyy-MM", 
            "yyyy/MM/dd", "yyyy/MM/dd HH:mm:ss", "yyyy/MM/dd HH:mm", "yyyy/MM",
            "yyyy.MM.dd", "yyyy.MM.dd HH:mm:ss", "yyyy.MM.dd HH:mm", "yyyy.MM"};

    /**
     * 获取当前Date型日期
     * 
     * @return Date() 当前日期
     */
    public static Date getNowDate()
    {
        return new Date();
    }

    /**
     * 获取当前日期, 默认格式为yyyy-MM-dd
     * 
     * @return String
     */
    public static String getDate()
    {
        return dateTimeNow(YYYY_MM_DD);
    }

    public static final String getTime()
    {
        return dateTimeNow(YYYY_MM_DD_HH_MM_SS);
    }

    public static final String dateTimeNow()
    {
        return dateTimeNow(YYYYMMDDHHMMSS);
    }

    public static final String dateTimeNow(final String format)
    {
        return parseDateToStr(format, new Date());
    }

    public static final String dateTime(final Date date)
    {
        return parseDateToStr(YYYY_MM_DD, date);
    }

    public static final String parseDateToStr(final String format, final Date date)
    {
        return new SimpleDateFormat(format).format(date);
    }

    public static final Date dateTime(final String format, final String ts)
    {
        try
        {
            return new SimpleDateFormat(format).parse(ts);
        }
        catch (ParseException e)
        {
            throw new RuntimeException(e);
        }
    }

    /**
     * 日期路径 即年/月/日 如2018/08/08
     */
    public static final String datePath()
    {
        Date now = new Date();
        return DateFormatUtils.format(now, "yyyy/MM/dd");
    }

    /**
     * 日期路径 即年/月/日 如20180808
     */
    public static final String dateTime()
    {
        Date now = new Date();
        return DateFormatUtils.format(now, "yyyyMMdd");
    }

    /**
     * 日期型字符串转化为日期 格式
     */
    public static Date parseDate(Object str)
    {
        if (str == null)
        {
            return null;
        }
        try
        {
            return parseDate(str.toString(), parsePatterns);
        }
        catch (ParseException e)
        {
            return null;
        }
    }

    /**
     * 获取服务器启动时间
     */
    public static Date getServerStartDate()
    {
        long time = ManagementFactory.getRuntimeMXBean().getStartTime();
        return new Date(time);
    }

    /**
     * 计算相差天数
     */
    public static int differentDaysByMillisecond(Date date1, Date date2)
    {
        return Math.abs((int) ((date2.getTime() - date1.getTime()) / (1000 * 3600 * 24)));
    }

    /**
     * 计算时间差
     *
     * @param endDate 最后时间
     * @param startTime 开始时间
     * @return 时间差（天/小时/分钟）
     */
    public static String timeDistance(Date endDate, Date startTime)
    {
        long nd = 1000 * 24 * 60 * 60;
        long nh = 1000 * 60 * 60;
        long nm = 1000 * 60;
        // long ns = 1000;
        // 获得两个时间的毫秒时间差异
        long diff = endDate.getTime() - startTime.getTime();
        // 计算差多少天
        long day = diff / nd;
        // 计算差多少小时
        long hour = diff % nd / nh;
        // 计算差多少分钟
        long min = diff % nd % nh / nm;
        // 计算差多少秒//输出结果
        // long sec = diff % nd % nh % nm / ns;
        return day + "天" + hour + "小时" + min + "分钟";
    }

    /**
     * 增加 LocalDateTime ==> Date
     */
    public static Date toDate(LocalDateTime temporalAccessor)
    {
        ZonedDateTime zdt = temporalAccessor.atZone(ZoneId.systemDefault());
        return Date.from(zdt.toInstant());
    }

    /**
     * 增加 LocalDate ==> Date
     */
    public static Date toDate(LocalDate temporalAccessor)
    {
        LocalDateTime localDateTime = LocalDateTime.of(temporalAccessor, LocalTime.of(0, 0, 0));
        ZonedDateTime zdt = localDateTime.atZone(ZoneId.systemDefault());
        return Date.from(zdt.toInstant());
    }

    /**
     * 时间戳+4位随机数
     */
    public static final String getOrderId()
    {
        // 获取当前时间戳
        long timestamp = System.currentTimeMillis();

        // 使用ThreadLocalRandom生成一个0到9999之间的随机数（包括0和9999）
        int randomNumber = ThreadLocalRandom.current().nextInt(10000);

        // 拼接时间戳和随机数，并确保随机数是4位数
        String timestampWithRandom = timestamp + String.format("%04d", randomNumber);

        return timestampWithRandom;
    }

    /**
     * 获取上个月（年月）
     */
    public static final String lastMonth()
    {
        LocalDate currentDate = LocalDate.now();
        LocalDate lastMonthDate = currentDate.minusMonths(1);

        DateTimeFormatter formatter = DateTimeFormatter.ofPattern(YYYY_MM);
        String lastMonth = lastMonthDate.format(formatter);

        return lastMonth;
    }

    /**
     * 获取当前季度的开始日期
     */
    public static final String startQuarterly()
    {
        // 获取当前日期
        LocalDate currentDate = LocalDate.now();

        // 计算当前季度
        int month = currentDate.getMonthValue();
        int startMonth, endMonth;

        if (month >= 1 && month <= 3) {
            startMonth = 1;
            endMonth = 3;
        } else if (month >= 4 && month <= 6) {
            startMonth = 4;
            endMonth = 6;
        } else if (month >= 7 && month <= 9) {
            startMonth = 7;
            endMonth = 9;
        } else {
            startMonth = 10;
            endMonth = 12;
        }

        // 构建开始和结束日期
        LocalDate startDate = LocalDate.of(currentDate.getYear(), startMonth, 1);
        LocalDate endDate = LocalDate.of(currentDate.getYear(), endMonth,
                endMonth == 6 || endMonth == 9 ? 30 : 31);
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern(YYYY_MM_DD);
        return startDate.format(formatter);
    }

    /**
     * 获取当前季度的结束日期
     */
    public static final String endQuarterly()
    {
        // 获取当前日期
        LocalDate currentDate = LocalDate.now();

        // 计算当前季度
        int month = currentDate.getMonthValue();
        int startMonth, endMonth;

        if (month >= 1 && month <= 3) {
            startMonth = 1;
            endMonth = 3;
        } else if (month >= 4 && month <= 6) {
            startMonth = 4;
            endMonth = 6;
        } else if (month >= 7 && month <= 9) {
            startMonth = 7;
            endMonth = 9;
        } else {
            startMonth = 10;
            endMonth = 12;
        }

        // 构建开始和结束日期
        LocalDate startDate = LocalDate.of(currentDate.getYear(), startMonth, 1);
        LocalDate endDate = LocalDate.of(currentDate.getYear(), endMonth,
                endMonth == 6 || endMonth == 9 ? 30 : 31);
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern(YYYY_MM_DD);
        return endDate.format(formatter);
    }

    /**
     * 获取传入年的所有月份（yyyy-mm）
     */
    public static final List<String> yearMonth(String year)
    {
        List<String> months = new ArrayList<>();
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern(YYYY_MM);

        for (int month = 1; month <= 12; month++) {
            YearMonth yearMonth = YearMonth.of(Integer.parseInt(year), month);
            months.add(yearMonth.format(formatter));
        }

        return months;
    }
}
